new;
library qam,optmum,pgraph;

rndseed 123;

cls;

/*
**> LN GMM
**
*/

n = 1000;
x_ln = exp(5 + rndn(n,1));
sv = 1|2;

proc H_LN(theta);
  local mu,sigma,m1,m2,M;

  mu = theta[1];
  sigma = sqrt(theta[2]^2);

  m1 = exp(mu + 0.5 * sigma^2);
  m2 = exp(2 * mu + sigma^2) * (exp(sigma^2) - 1);

  M = zeros(n,2);
  M[.,1] = x_ln - m1;
  M[.,2] = (x_ln - m1)^2 - m2;

  retp(M);
endp;

{theta,stderr,Mcov,Qmin} = regGMM(0,0,&H_LN,sv,0,0);

/*

===============================================================================
 regGMM - Generalized Method of Moments                   10/12/2011  11:10 am
===============================================================================


Total observations:                                         1000  
Missing observations:                                          0  
Usable observations:                                        1000  
Valid cases:                                                1000  

Number of parameters:                                          2  
Number of unrestricted parameters:                             2  
Number of moments:                                             2  
Degrees of freedom:                                          998  

Value of the criterion function:                         0.00000  


Parameters      estimates       std.err.      t-statistic         p-value   
--------------------------------------------------------------------------
P01             5.057032        0.038121       132.657837        0.000000
P02             0.917197        0.036394        25.201840        0.000000

Covariance matrix: inverse of the negative Hessian.

*/


/*
**> Truncated Normal Distribution
**
*/

n = 1000000;
ns = 1;

mu = 2;
sigma = 3;
x_star = 1;

alpha = (x_star - mu) ./ sigma;
Phi = cdfn(alpha);
lambda = pdfn(alpha) ./ (1 - Phi);
delta = lambda .* (lambda - alpha);

U = rndu(n,ns);
X = mu + sigma .* cdfni(Phi + U .* (1 - Phi));

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pbartyp = {6 1};
  xtics(0,15,1,0);
  xlabel(" "); ylabel(" ");
  graphprt("-c=1 -cf=ex4-1.eps");
  call histp(X,50);

empirical_moments = meanc(X)~meanc(X^2)~stdc(X);

m1 = mu + sigma .* lambda;
m2 = mu^2 + 2 * mu .* sigma .* lambda + (sigma^2) .* (1 + alpha .* lambda);
m3 = sigma .* sqrt(1 - delta);

theoretical_moments = m1'~m2'~m3';

print "Truncated case:";
print ftosa(theoretical_moments,"%lf",4,5);
print ftosa(empirical_moments,"%lf",4,5);
print "--------------------------------------";

/*
**> Censured Normal Distribution
**
*/

y_star = x_star;

beta = (y_star - mu) ./ sigma;

X = mu + sigma .* rndn(n,ns);
c = (X .> x_star);
Y = X .* c + y_star .* (1 - c);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pbartyp = {6 4};
  xtics(0,15,1,0);
  xlabel(" "); ylabel(" ");
  graphprt("-c=1 -cf=ex4-2.eps");
  call histp(Y,50);

empirical_moments = meanc(Y)~meanc(Y^2)~stdc(Y);

m1 = Phi .* y_star +  (1 - Phi) .* (mu + sigma .* lambda);
// m2 = Phi .* (y_star^2) + (1 - Phi) .* (mu^2 + 2 * mu .* sigma .* lambda + 
//      (sigma^2) .* (1 + alpha .* lambda));
m2 = Phi .* (y_star^2) + (1 - Phi) .* mu^2 + 2 .* mu .* sigma .* pdfn(alpha) + 
     (sigma^2) .* (1 - Phi + alpha .* pdfn(alpha));
m1_sqr = Phi^2 .* y_star^2 +  (1 - Phi)^2 .* mu^2 + 
         2 * mu .* sigma .* (1 - Phi) .* pdfn(alpha) +
         sigma^2 .* pdfn(alpha)^2 + 2 .* Phi .* (1 - Phi) .* y_star .* mu + 
         2 .* Phi .* y_star .* sigma .* pdfn(alpha);

// First expression of the variance
m3 = sqrt(m2-m1^2);

// Second expression of the variance
m3 = Phi .* (1 - Phi) .* y_star^2 + Phi .* (1 - Phi) .* mu^2 + 
     2 * Phi .* pdfn(alpha) .* mu .* sigma + 
     pdfn(alpha) .* (alpha - pdfn(alpha)) .* sigma^2 + (1 - Phi) .* sigma^2 - 
     2 * Phi .* (1 - Phi) .* y_star .* mu - 2 * Phi .* pdfn(alpha) .* y_star .* sigma ;
m3 = sqrt(m3);

// third expression of the variance
m3 = Phi .* (1 - Phi) .* beta^2  .* sigma^2 - 2 * Phi .* pdfn(alpha) .* beta .* sigma^2 + 
     sigma^2 .* (1 - Phi) .* (1 + pdfn(alpha) .* (alpha - pdfn(alpha)) ./ (1 - Phi) );
m3 = Phi .* (1 - Phi) .* beta^2  .* sigma^2 - 2 * Phi .* pdfn(alpha) .* beta .* sigma^2 + 
     sigma^2 .* (1 - Phi) .* (1 + lambda .* (alpha - pdfn(alpha)) );
m3 = sigma^2 .* (1 - Phi) .* ( ( 1 -delta) + (beta - lambda)^2 .* Phi );
m3 = sqrt(m3);

theoretical_moments = m1'~m2'~m3';

print "Censored case:";
print ftosa(theoretical_moments,"%lf",4,5);
print ftosa(empirical_moments,"%lf",4,5);
print "--------------------------------------";

/*
Truncated case:
         3.79547         18.38641          1.99520 
         3.79588         18.39650          1.99695 
--------------------------------------
Censored case:
         2.76271         11.96315          2.08101 
         2.76213         11.96248          2.08161 
--------------------------------------
*/


/*
**> TOBIT GMM
**
*/

rndseed 123456;

n = 1000;

x_tobit = rndu(n,1);
beta = 1;
u = 0.5 * rndn(n,1);
y = beta * x_tobit + u;
y_tobit = y .* (y .> 0);

sv = 1|1;

proc H_TOBIT_GMM(theta);
  local beta,sigma,alpha,mu,Phi,lambda,delta,m1,m2,M;

  beta = theta[1];
  sigma = sqrt(theta[2]^2);

  mu = beta .* x_tobit;
  alpha = - mu ./ sigma;
  Phi = cdfn(alpha);
  lambda = pdfn(alpha) ./ (1 - Phi);
  delta = lambda .* (lambda - alpha);

  m1 = Phi .* 0 +  (1 - Phi) .* (mu + sigma .* lambda);
  m2 = sigma^2 .* (1 - Phi) .* ( ( 1 - delta) + (alpha - lambda)^2 .* Phi );

  M = zeros(n,2);
  M[.,1] = y_tobit - m1;
  M[.,2] = (y_tobit - m1)^2 - m2;

  retp(M);
endp;

{theta,stderr,Mcov,Qmin} = regGMM(0,0,&H_TOBIT_GMM,sv,0,0);

/*

===============================================================================
 regGMM - Generalized Method of Moments                   10/12/2011  11:07 am
===============================================================================


Total observations:                                         1000  
Missing observations:                                          0  
Usable observations:                                        1000  
Valid cases:                                                1000  

Number of parameters:                                          2  
Number of unrestricted parameters:                             2  
Number of moments:                                             2  
Degrees of freedom:                                          998  

Value of the criterion function:                         0.00000  


Parameters      estimates       std.err.      t-statistic         p-value   
--------------------------------------------------------------------------
P01             1.055653        0.028710        36.770126        0.000000
P02             0.481552        0.012119        39.736093        0.000000

Covariance matrix: inverse of the negative Hessian.

*/


/*
**> TOBIT SMM
**
*/

proc H_TOBIT_SMM(theta);
  local beta,sigma,y_smm,y_tobit_smm,m1,m2,M;

  beta = theta[1];
  sigma = sqrt(theta[2]^2);

  rndseed 123;
  y_smm = beta .* x_tobit + sigma * rndn(n,1);
  y_tobit_smm = y_smm .* (y_smm .> 0);

  m1 = meanc(y_tobit_smm);
  m2 = stdc(y_tobit_smm)^2;

  M = zeros(n,2);
  M[.,1] = y_tobit - m1;
  M[.,2] = (y_tobit - m1)^2 - m2;

  retp(M);
endp;

{theta,stderr,Mcov,Qmin} = regGMM(0,0,&H_TOBIT_SMM,sv,0,0);

/*

===============================================================================
 regGMM - Generalized Method of Moments                   10/12/2011  11:07 am
===============================================================================


Total observations:                                         1000  
Missing observations:                                          0  
Usable observations:                                        1000  
Valid cases:                                                1000  

Number of parameters:                                          2  
Number of unrestricted parameters:                             2  
Number of moments:                                             2  
Degrees of freedom:                                          998  

Value of the criterion function:                         0.00000  


Parameters      estimates       std.err.      t-statistic         p-value   
--------------------------------------------------------------------------
P01             1.013257        0.037996        26.667744        0.000000
P02             0.533073        0.020113        26.503564        0.000000

Covariance matrix: inverse of the negative Hessian.

*/